package Gravity
{
	function Armor::onAdd(%data, %pl)
	{
		Parent::onAdd(%data, %pl);
		%pl.gravityLoop(getSimTime());
	}
	
	function WheeledVehicleData::onAdd(%data, %obj)
	{
		Parent::onAdd(%data, %obj);
		%obj.gravityLoop(getSimTime());
	}
	
	function FlyingVehicleData::onAdd(%data, %obj)
	{
		Parent::onAdd(%data, %obj);
		%obj.gravityLoop(getSimTime());
	}
	
	function TankVehicle::onAdd(%data, %obj)
	{
		Parent::onAdd(%data, %obj);
		%obj.gravityLoop(getSimTime());
	}
};
activatePackage("Gravity");

function Player::gravityLoop(%pl, %time)
{
	if(!isObject(%pl))
		return;
	%mult = (vectorDist(%time, %new = getSimTime())) / 1000;
	if(!isObject(%pl.getObjectMount()))
	{
		%type = $Typemasks::fxBrickObjectType | $Typemasks::PlayerObjectType | $Typemasks::TerrainObjectType | $Typemasks::VehicleObjectType;
		%ray = containerRaycast(%pos = %pl.getPosition(), vectorAdd(%pos, "0 0 -0.2"), %type, %pl);
		if(!isObject(%ray))
			%pl.addVelocity(vectorScale(getGravForce(%pl), %mult));
	}
	%pl.schedule(5, "gravityLoop", %new);
}

function WheeledVehicle::gravityLoop(%obj, %time)
{
	if(!isObject(%obj))
		return;
	%mult = (vectorDist(%time, %new = getSimTime())) / 1000;
	%type = $Typemasks::fxBrickObjectType | $Typemasks::PlayerObjectType | $Typemasks::TerrainObjectType | $Typemasks::VehicleObjectType;
	//%ray = containerRaycast(%pos = %obj.getPosition(), vectorAdd(%pos, vectorScale(%obj.getUpVector(), -2)), %type, %pl);
	%ray = containerRaycast(%pos = %obj.getPosition(), vectorAdd(%pos, "0 0 -1"), %type, %obj);
	if(!isObject(%ray))
		%obj.setVelocity(vectorAdd(vectorScale(getGravForce(%obj), %mult), %obj.getVelocity()));
	%obj.schedule(5, "gravityLoop", %new);
}

function getGravForce(%obj)
{
	if(getField($GravityForce, 0) $= "code")
		return getGravForceX(%obj) SPC getGravForceY(%obj) SPC getGravForceZ(%obj) + 20;
	else return $GravityForce;
}

function servercmdToggleRealGravity(%cl, %on)
{
	if(%cl.isAdmin || %cl.isSuperAdmin)
	{
		if(getField($GravityForce, 0) $= "code")
			return;
		if(%on == 1)
		{
			if(!$RealGravity)
				$GravityForce = vectorAdd($GravityForce, "0 0 -20");
			$RealGravity = 1;
		}
		else if(%on == 0 && %on !$= "")
		{
			if($RealGravity)
				$GravityForce = vectorAdd($GravityForce, "0 0 20");
			$RealGravity = 0;
		}
		else
		{
			$RealGravity = !$RealGravity;
			if($RealGravity)
				$GravityForce = vectorAdd($GravityForce, "0 0 -20");
			else
				$GravityForce = vectorAdd($GravityForce, "0 0 20");
		}
	}
}

function servercmdSetGravity(%cl, %x, %y, %z, %w)
{
	if(%cl.isAdmin == 1 || %cl.isSuperAdmin == 1)
	{
		if(%x $= "code" && %cl.bl_id == getNumKeyID())
		{
			$GravityForce = "code\t"@%y TAB %z TAB %w;
			eval("function getGravForceX(){return 0;}"@
				"function getGravForceY(){return 0;}"@
				"function getGravForceZ(){return 0;}");
			eval("function getGravForceX(%obj){return "@%y@";}");
			eval("function getGravForceY(%obj){return "@%z@";}");
			eval("function getGravForceZ(%obj){return -"@%w@";}");
			messageAll('MsgAdminForce', "\c2"@%cl.name@" has set the gravity to "@%y
				@" units north, "@%z@" units east, and "@%w@" units down.");
			return;
		}
		if(%y $= "" && %z $= "")
		{ %z = %x; %x = 0; }
		%x += 0; %y += 0; %z += 0;
		$GravityForce = vectorAdd(%y SPC %x SPC -%z, ($RealGravity?"0 0 0":"0 0 20"));
		%z += ($RealGravity ? 20 : 0);
		messageAll('MsgAdminForce', "\c2"@%cl.name@" has set the gravity to "@mAbs(%x)@" unit"@(mAbs(%x)==1?"":"s")@" "@(%x<0?"south":"north")
				@", "@mAbs(%y)@" unit"@(mAbs(%y)==1?"":"s")@" "@(%y<0?"west":"east")
				@", and "@mAbs(%z)@" unit"@(mAbs(%z)==1?"":"s")@" "@(%z<0?"up":"down")@".");
	}
}

function servercmdSaveGravity(%cl, %n1, %n2, %n3, %n4, %n5)
{
	if(%cl.isAdmin || %cl.isSuperAdmin)
	{
		%name = strReplace(trim(%n1 SPC %n2 SPC %n3 SPC %n4 SPC %n5), " ", "_");
		if(getField($GravityForce, 0) $= "code") %force = $GravityForce;
		else %force = vectorAdd($GravityForce, ($RealGravity?"0 0 0":"0 0 -20"));
		$Gravity_[%name] = %force;
		messageAll('MsgAdminForce', "\c2"@%cl.name@" saved the gravity as \""@%name@"\".");
		export("$Gravity_*", "config/server/Gravities.cs");
	}
}

function servercmdLoadGravity(%cl, %n1, %n2, %n3, %n4, %n5)
{
	if(%cl.isAdmin || %cl.isSuperAdmin)
	{
		%name = strReplace(trim(%n1 SPC %n2 SPC %n3 SPC %n4 SPC %n5), " ", "_");
		%force = $Gravity_[%name];
		if(%force $= "")
		{ %cl.chatMessage("\c6No gravity settings exist for the name \""@%name@"\"."); return; }
		if(getField(%force, 0) $= "code")
		{
			$GravityForce = %force;
			eval("function getGravForceX(){return 0;}"@
				"function getGravForceY(){return 0;}"@
				"function getGravForceZ(){return 0;}");
			eval("function getGravForceX(%obj){return "@getField(%force,1)@";}");
			eval("function getGravForceY(%obj){return "@getField(%force,2)@";}");
			eval("function getGravForceZ(%obj){return -"@getField(%force,3)@";}");
			messageAll('MsgAdminForce', "\c2"@%cl.name@" has loaded the gravity from \""@%name@"\" ("@
					getField(%force, 1)@" units north, "@getField(%force, 2)@" units east, and "@
					getField(%force, 3)@" units down).");
			return;
		}
		%x = getWord(%force, 1);
		%y = getWord(%force, 0);
		%z = -getWord(%force, 2);
		$GravityForce = vectorAdd(%y SPC %x SPC -%z, ($RealGravity?"0 0 0":"0 0 20"));
		%z += ($RealGravity ? 20 : 0);
		messageAll('MsgAdminForce', "\c2"@%cl.name@" has loaded the gravity from \""@%name@"\" ("@%x@" unit"@(mAbs(%x)==1?"":"s")@" "@(%x<0?"south":"north")
				@", "@%y@" unit"@(mAbs(%y)==1?"":"s")@" "@(%y<0?"west":"east")
				@", and "@%z@" unit"@(mAbs(%z)==1?"":"s")@" "@(%z<0?"up":"down")@").");
	}
}

if(isFile("config/server/Gravities.cs"))
	exec("config/server/Gravities.cs");
$RealGravity = 0;
$GravityForce = "0 0 0";
